; Fire One 2600
;
; 2600 Programming For Newbies project
;
; Compiles with DASM
;
; Build command:
; dasm fireone.s -f3 -lfire.txt -ofire.bin -v3
;
; History
;
; 07.03.2K4     - Started
; 07.03.2K4     - Bankswitching skeleton provided by MR
; 12.03.2K4     - Added background colors for main display by DR
; 13.03.2K4     - VBLANK code added by MR

    processor  6502
    include    vcs.h
    include    macro.h

;----------------------------------------------------------
; 4K GAME
;----------------------------------------------------------

; Compile switches

NTSC                    = 0
PAL                     = 1
COMPILE_VERSION         = PAL

VISIBLE_LINES           = 191

;; The main display starts on line 62 and is 108 lines.
;; Since we count down, the first scanline is at (192 - 62) + 1
;; The last scanline is the first scanline - 108
START_MAIN_DISP         = 131
END_MAIN_DISP           = 23
SKY_SLICE_HEIGHT        = 5

;-----------------------
; NTSC dependent defines
;-----------------------
    IF COMPILE_VERSION == NTSC
OCEAN_COLOR             = $80
FIRST_SKY_COLOR         = $A0
SECOND_SKY_COLOR        = $86
THIRD_SKY_COLOR         = $7A
FOURTH_SKY_COLOR        = $7E

VBLANK_TIMER            = 44
OVERSCAN_TIMER          = 37

;-----------------------
; PAL dependent defines
;-----------------------
    ELSE

OCEAN_COLOR             = $D0
FIRST_SKY_COLOR         = $90
SECOND_SKY_COLOR        = $92
THIRD_SKY_COLOR         = $96
FOURTH_SKY_COLOR        = $9A

VBLANK_TIMER            = 75
OVERSCAN_TIMER          = 65
    ENDIF



    SEG.U vars
    ORG $80

; Both Banks:
gameState       ds 1
bcdScore        ds 1
frameCounter    ds 1
rnd             ds 1
tempVar1        ds 1
tempVar2        ds 1
tempVar3        ds 1
tempVar4        ds 1

;---------------
; Main Display
;---------------

mainVariable1   ds 1

;---------------
; State Display
;---------------

stateVariable1  ds 1

;---------------
; Sonar
;---------------

sonarVariable   ds 1


    SEG     Bank0
    ORG     $1000
    RORG    $D000

MainGame

;------------------
; Main Frame Loop
;------------------

FrameStart
    VERTICAL_SYNC

    ;------------------
    ; Vertical Blank
    ;------------------

    ;; Setup timer
    LDA #VBLANK_TIMER
    STA TIM64T

    ;; Use X as scanline counter
    LDX #VISIBLE_LINES

    ;; Now wait for the end of vertical blank
.vblankloop
    LDA INTIM
    BNE .vblankloop



    ;------------------
    ; Visible Scanlines
    ;------------------

    ;; This is the last WSYNC of the vertical blank. If a subroutine
    ;; is used to start display on the first scanline, this could
    ;; be moved to the subroutine to allow it a little extra time
    ;; to prepare
    STA WSYNC
    STA VBLANK  ; Stop vertical blank


.scanlineLoop
    ;; Check for the main display. Note this jumps to the main
    ;; display on the scanline before the display. This gives the
    ;; MainDisplayCode a chance to do some work before starting its
    ;; first scanline
    CPX #START_MAIN_DISP+1
    BNE .notmaindisp
    JSR MainDisplayCode
.notmaindisp

    STA WSYNC
    DEX
    BNE .scanlineLoop

    ;------------------
    ; Overscan
    ;------------------

    ; Blank the screen
    LDA $02
    STA VBLANK          ; Start vertical blank

    ;; Setup timer
    LDA #OVERSCAN_TIMER
    STA TIM64T

    ;; Now wait for the end of the overscan
.overscanloop
    LDA INTIM
    BNE .overscanloop

    JMP FrameStart

;------------------
; Main Display code
;------------------

MainDisplayCode SUBROUTINE
.mainDisplayLoop
    LDA #FIRST_SKY_COLOR
    LDY #SKY_SLICE_HEIGHT
    JSR DisplayMainStripe

    LDA #SECOND_SKY_COLOR
    LDY #SKY_SLICE_HEIGHT
    JSR DisplayMainStripe

    LDA #THIRD_SKY_COLOR
    LDY #SKY_SLICE_HEIGHT
    JSR DisplayMainStripe

    LDA #FOURTH_SKY_COLOR
    LDY #SKY_SLICE_HEIGHT
    JSR DisplayMainStripe

    LDA #OCEAN_COLOR
    LDY #((START_MAIN_DISP - END_MAIN_DISP)- (4 * SKY_SLICE_HEIGHT))
    JSR DisplayMainStripe

    ;; Restore background color to black for next scanline
    LDA #0
    DEX
    STA WSYNC
    STA COLUBK
    RTS

;; SUBROUTINE: DisplayMainStripe
;; Displays a stripe of the main display
;; Arguments:
;; A: Background color for stripe
;; Y: Height in scanlines of stripe
;; Destroys:
;; A, Y, N, Z flags
;;
;; Notes:
;; Keeps X equal to the current scanline (i.e. decrements X for every WSYNC)
;; Expects to be called on the scanline before it starts displaying
;; Returns before last scanline has finished (caller must call WSYNC)
DisplayMainStripe SUBROUTINE
   ;; Set background color as first part of the scanline
   STA WSYNC
   STA COLUBK
   DEY
   DEX

   ;; Call WSYNC for all but the last line of the slice
.loop
   STA WSYNC
   DEX
   DEY
   BNE .loop
   RTS


;-------------------
; State Display code
;-------------------



;------------
; Sonar code
;------------

    ORG $1FEC
    RORG $DFEC

FirstInit
    BIT $1FF9
    NOP             ; dummy

SwitchToBank1:
    BIT $1FF9
    JMP MainGame

    .word $FFFF, $FFFF  ; bank-switching hotspots
    .word FirstInit
    .word FirstInit
    .word FirstInit

    SEG.U   variables
    ORG     $80

; Both Banks:
gameState       ds 1
bcdScore        ds 1
frameCounter    ds 1
rnd             ds 1
tempVar1        ds 1
tempVar2        ds 1
tempVar3        ds 1
tempVar4        ds 1

;-------
; Intro
;-------

introVariable1  ds 1

;---------
; Network
;---------

netVariable1    ds 1

    SEG     Bank1
    ORG     $2000
    RORG    $F000

start
    CLEAN_START

Intro
    JMP SwitchToBank0

;------------
; Intro code
;------------


;--------------
; Network code
;--------------

    ORG     $2FEF
    RORG    $FFEF

    BRK                     ; coming from the other bank

SwitchToBank0
    BIT $1FF8
    JMP Intro

    .word $FFFF, $FFFF      ; bank-switching hotspots
    .word start
    .word start
    .word start